package sample.biz.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import sample.biz.dao.PetDao;
import sample.biz.domain.Pet;
import sample.biz.exception.BussinessException;
import sample.biz.service.PetService;

@Service
public class PetServiceImpl implements PetService {
    
    @Autowired
    private PetDao petDao;
    
    @Transactional(
            propagation=Propagation.REQUIRED, 
            isolation=Isolation.READ_COMMITTED, 
            timeout=10, readOnly=false, 
            rollbackFor = BussinessException.class )
    public void updatePet(Pet pet) throws BussinessException {
        petDao.updatePet(pet);
    }
    
    
    @Autowired
    private PlatformTransactionManager txManager;

    public void updatePetProgrammaticTransaction(Pet pet) {
        DefaultTransactionDefinition def = new DefaultTransactionDefinition();
        def.setPropagationBehavior(TransactionDefinition.PROPAGATION_REQUIRED);
        def.setIsolationLevel(TransactionDefinition.ISOLATION_READ_COMMITTED);
        def.setTimeout(10);
        def.setReadOnly(false);
        TransactionStatus status = txManager.getTransaction(def);
        try {
            petDao.updatePet(pet);
        } catch (RuntimeException e) {
            txManager.rollback(status);
            throw e;
        }
        txManager.commit(status);
    }
}
